@@ -1,7 +1,9 @@
 require "test_helper"
 
 class TestUnload < LoaderTest
-  test "unload removes all autoloaded constants" do
+  module Namespace; end
+
+  test "unload removes all autoloaded constants (Object)" do
     files = [
       ["user.rb", "class User; end"],
       ["admin/root.rb", "class Admin::Root; end"]
@@ -19,6 +21,24 @@ class TestUnload < LoaderTest
     end
   end
 
+  test "unload removes all autoloaded constants (Namespace)" do
+    files = [
+      ["user.rb", "class #{Namespace}::User; end"],
+      ["admin/root.rb", "class #{Namespace}::Admin::Root; end"]
+    ]
+    with_setup(files, namespace: Namespace) do
+      assert Namespace::User
+      assert Namespace::Admin::Root
+      admin = Namespace::Admin
+
+      loader.unload
+
+      assert !Namespace.const_defined?(:User)
+      assert !Namespace.const_defined?(:Admin)
+      assert !admin.const_defined?(:Root)
+    end
+  end
+
   test "unload removes autoloaded constants, even if #name is overridden" do
     files = [["x.rb", <<~RUBY]]
       module X
